@@ -58,13 +58,7 @@ const (
 func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) error {
 	decoder := xml.NewDecoder(reader)
 	decoder.Strict = false
-	decoder.CharsetReader = func(charset string, input io.Reader) (io.Reader, error) {
-		e, err := ianaindex.MIME.Encoding(charset)
-		if err != nil {
-			return nil, err
-		}
-		return transform.NewReader(input, e.NewDecoder()), nil
-	}
+	decoder.CharsetReader = getCharsetReader
 
 	level := 0
 	hasContent := false
@@ -169,7 +163,9 @@ func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) er
 				_, _ = fmt.Fprint(writer, newline)
 			}
 		case xml.EndElement:
-			level--
+			if level > 0 {
+				level--
+			}
 			currentTagName := getTokenFullName(typedToken.Name, nsAliases)
 			if !hasContent {
 				if lastTagName != currentTagName {
@@ -211,7 +207,8 @@ func XPathQuery(reader io.Reader, writer io.Writer, query string, singleNode boo
 
 	doc, err := xmlquery.ParseWithOptions(reader, xmlquery.ParserOptions{
 		Decoder: &xmlquery.DecoderOptions{
-			Strict: false,
+			Strict:        false,
+			CharsetReader: getCharsetReader,
 		},
 	})
 	if err != nil {
@@ -380,7 +377,9 @@ func FormatHtml(reader io.Reader, writer io.Writer, indent string, colors int) e
 				forceNewLine = false
 			}
 		case html.EndTagToken:
-			level--
+			if level > 0 {
+				level--
+			}
 			tagName, _ := tokenizer.TagName()
 
 			if forceNewLine {
@@ -454,7 +453,9 @@ func FormatJson(reader io.Reader, writer io.Writer, indent string, colors int) e
 				level++
 				suffix = strings.Repeat(indent, level)
 			case '}':
-				level--
+				if level > 0 {
+					level--
+				}
 				_, _ = fmt.Fprint(writer, newline, strings.Repeat(indent, level), tagColor("}"))
 				if tokenState == jsonTokenArrayComma {
 					suffix = "," + newline + strings.Repeat(indent, level)
@@ -464,7 +465,9 @@ func FormatJson(reader io.Reader, writer io.Writer, indent string, colors int) e
 				level++
 				suffix = strings.Repeat(indent, level)
 			case ']':
-				level--
+				if level > 0 {
+					level--
+				}
 				_, _ = fmt.Fprint(writer, newline, strings.Repeat(indent, level), tagColor("]"))
 			}
 		case string:
@@ -603,3 +606,14 @@ func normalizeSpaces(input string, indent string, level int) string {
 
 	return input
 }
+
+func getCharsetReader(charset string, input io.Reader) (io.Reader, error) {
+	if strings.ToLower(charset) == "utf-16" {
+		charset = "utf-8"
+	}
+	e, err := ianaindex.MIME.Encoding(charset)
+	if err != nil {
+		return nil, err
+	}
+	return transform.NewReader(input, e.NewDecoder()), nil
+}
